from abc import ABCMeta, abstractproperty, abstractmethod
import json
from django.conf import settings
from .utils import DefaultConnection, PushLogger, APP_PREFIX
from django.db import models


class SNSCRUDMixin(object):
    __metaclass__ = ABCMeta

    def __init__(self, resource_name):
        super(SNSCRUDMixin, self).__init__()
        self.resource_name = resource_name
        self.arn = None

    def __eq__(self, other):
        if type(other) is type(self):
            return self.arn == other.arn
        return False

    def __ne__(self, other):
        return not self.__eq__(other)

    @property
    def response_key(self):
        return u'Create{0}Response'.format(self.resource_name)

    @property
    def result_key(self):
        return u'Create{0}Result'.format(self.resource_name)

    @property
    def arn_key(self):
        return u'{0}Arn'.format(self.resource_name)

    @property
    def is_registered(self):
        return self.arn is not None

    def set_arn_from_response(self, response_dict):
        """
        :type response_dict: dict
        :param response_dict:
        :rtype boolean:
        :return :
        """
        success = False
        try:
            self.arn = response_dict[self.response_key][self.result_key][self.arn_key]
            success = True
        except KeyError:
            pass
        return success

    @abstractmethod
    def register(self, connection=None):
        """

        :param connection:
        :rtype boolean:
        :return: if create was successful
        """
        pass


class SNSDevice(SNSCRUDMixin):
    def __init__(self, platform, token, arn=None, is_enabled=False):
        """
        :type platform: SNSPlatformApplication
        :param platform:
        :type token: unicode
        :param token:
        :return:
        """
        super(SNSDevice, self).__init__("PlatformEndpoint")
        self.token = token
        self.platform = platform
        self.is_enabled = is_enabled
        self.arn = arn

    @property
    def arn_key(self):
        return "EndpointArn"

    @DefaultConnection
    def register(self, custom_user_data=u"", connection=None):
        if not self.platform.is_registered:
            success = self.platform.register(connection)
            if not success:
                return success
        response = connection.create_platform_endpoint(self.platform.arn, self.token, custom_user_data=custom_user_data)
        self.is_enabled = success = self.set_arn_from_response(response)
        if self.is_registered:
            self.platform.app_topic.register_device(self)
        return success

    @DefaultConnection
    def register_or_update(self, new_token=None, custom_user_data=u"", connection=None):
        if self.is_registered:
            result = self.update(new_token, custom_user_data, connection)
        else:
            result = self.register(custom_user_data, connection)
        return result

    @DefaultConnection
    def delete(self, connection=None):
        """
        :type connection: SNSConnection
        :param connection: the connection which should be used. if the argument isn't set there will be created a default connection
        :return:
        """
        if not self.is_registered:
            raise SNSNotCreatedException

        return connection.delete_endpoint(self.arn)

    @DefaultConnection
    def send_message(self, message, connection=None):
        return connection.publish(message=message, target_arn=self.arn)

    @PushLogger
    @DefaultConnection
    def send(self, push_message, connection=None):
        """
        :type connection: SNSConnection
        :param connection: the connection which should be used. if the argument isn't set there will be created a default connection
        :return:
        """
        push_message = self.platform.format_payload(push_message)
        json_string = json.dumps(push_message)
        return connection.publish(message=json_string, target_arn=self.arn, message_structure="json")


    @DefaultConnection
    def update(self, new_token=None, custom_user_data=u"", connection=None):
        """
        :type connection: SNSConnection
        :param connection: the connection which should be used. if the argument isn't set there will be created a default connection
        :return:
        """

        new_token = new_token if new_token else self.token
        attributes = {"Enabled": True, "Token": new_token}
        if custom_user_data:
            attributes["CustomUserData"] = custom_user_data
        answer = connection.set_endpoint_attributes(self.arn, attributes)
        self.is_enabled = True
        self.token = new_token
        return answer

    def sign(self, push_message):
        push_message.receiver_arn = self.arn
        push_message.message_type = PushMessage.MESSAGE_TYPE_TOPIC

    @classmethod
    def create_from_endpoints(cls, platform, endpoints):
        devices = list()
        for endpoint in endpoints:
            attributes = endpoint[u'Attributes']
            token = attributes[u'Token']
            is_enabled = attributes[u"Enabled"].lower() == u'true'
            arn = endpoint[u'EndpointArn']
            device = cls(platform, token)
            device.arn = arn
            device.is_enabled = is_enabled
            devices.append(device)
        return devices


class SNSPlatformApplication(SNSCRUDMixin):
    __metaclass__ = ABCMeta

    def __init__(self, app_name=APP_PREFIX):
        super(SNSPlatformApplication, self).__init__("PlatformApplication")
        self.app_name = app_name
        self._app_topic = None

    @property
    def name(self):
        return u"_".join([self.app_name, self.platform]).lower()

    @property
    def app_topic(self):
        if not self._app_topic:
            topic = Topic(self.app_name)
            if topic.register():
                self._app_topic = topic
        return self._app_topic

    @abstractproperty
    def platform(self):
        """
        :rtype: unicode
        :return: the platform name
        """
        pass

    @abstractproperty
    def credential(self):
        pass

    @abstractproperty
    def principal(self):
        pass

    @property
    def attributes(self):
        return {"PlatformCredential": self.credential,
                "PlatformPrincipal": self.principal
        }

    @DefaultConnection
    def register(self, connection=None):
        """
        Adds an app to SNS. Apps are per platform. The name of a sns application is app_platform

        :type connection: SNSConnection
        :param connection: the connection which should be used. if the argument isn't set there will be created a default connection
        :rtype bool:
        :return:
        """

        response = connection.create_platform_application(self.name, self.platform, self.attributes)
        return self.set_arn_from_response(response)

    @DefaultConnection
    def delete(self, connection=None):
        """
        :type connection: SNSConnection
        :param connection: the connection which should be used. if the argument isn't set there will be created a default connection
        :return:
        """
        if not self.is_registered:
            raise SNSNotCreatedException
        return connection.delete_platform_application(self.arn)

    @DefaultConnection
    def all_devices(self, connection=None):
        devices_list = list()

        def get_next(nexttoken):
            response = connection.list_endpoints_by_platform_application(platform_application_arn=self.arn,
                                                                         next_token=nexttoken)
            result = response[u'ListEndpointsByPlatformApplicationResponse'][
                u'ListEndpointsByPlatformApplicationResult']
            devices = result[u'Endpoints']
            devices_list.extend(SNSDevice.create_from_endpoints(self, devices))
            return result[u'NextToken']

        next_token = get_next(None)

        while next_token:
            next_token = get_next(next_token)

        return devices_list

    def format_payload(self, data):
        return {self.platform: json.dumps(data)}


class APNApplication(SNSPlatformApplication):
    @property
    def platform(self):
        return settings.SCARFACE_APNS_MODE

    @property
    def credential(self):
        return settings.SCARFACE_APNS_PRIVATE_KEY

    @property
    def principal(self):
        return settings.SCARFACE_APNS_CERTIFICATE

    def format_payload(self, message):
        """
        :type message: PushMessage
        :param message:
        :return:
        """

        payload = format_push(message.badge_count, message.context, message.context_id, message.has_new_content,
                              message.message, message.sound)
        if message.extra_payload:
            payload.update(message.extra_payload)
        return super(APNApplication, self).format_payload(payload)


class GCMApplication(SNSPlatformApplication):
    @property
    def platform(self):
        return u"GCM"

    @property
    def credential(self):
        return settings.SCARFACE_GCM_API_KEY

    @property
    def principal(self):
        return None

    def format_payload(self, message):
        """
        :type data: PushMessage
        :param data:
        :return:
        """
        data = message.as_dict()
        h = hash(frozenset(data.items()))
        return super(GCMApplication, self).format_payload({"collapse_key": h, "data": data})


class Topic(SNSCRUDMixin):
    def __init__(self, name):
        super(Topic, self).__init__("Topic")
        self.name = name

    @DefaultConnection
    def register(self, connection=None):
        response = connection.create_topic(self.name)
        return self.set_arn_from_response(response)

    @DefaultConnection
    def delete(self, connection=None):
        if not self.is_registered:
            raise SNSNotCreatedException
        return connection.delete_topic(self.arn)

    @DefaultConnection
    def register_device(self, device, connection=None):
        """
        :type device: SNSDevice
        :param device:
        :type connection: SNSConnection
        :param connection: the connection which should be used. if the argument isn't set there will be created a default connection
        :rtype bool:
        :return:
        """
        if not (self.is_registered and device.is_registered):
            raise SNSNotCreatedException
        success = connection.subscribe(topic=self.arn, endpoint=device.arn, protocol="application")
        return success

    @DefaultConnection
    def all_subscriptions(self, connection=None):
        subscriptions_list = list()

        def get_next(nexttoken):
            response = connection.get_all_subscriptions_by_topic(topic=self.arn, next_token=nexttoken)
            result = response["ListSubscriptionsByTopicResponse"]["ListSubscriptionsByTopicResult"]
            subs = result[u'Subscriptions']
            subscriptions_list.extend(subs)
            return result[u'NextToken']

        next_token = get_next(None)

        while next_token:
            next_token = get_next(next_token)

        return subscriptions_list

    def sign(self, push_message):
        push_message.receiver_arn = self.arn
        push_message.message_type = PushMessage.MESSAGE_TYPE_TOPIC

    @PushLogger
    @DefaultConnection
    def send(self, push_message, platforms, connection=None):
        """

        :type push_message: PushMessage
        :param push_message:
        :type platforms:list
        :param platforms:
        :return:
        """
        payload = dict()
        for platform in platforms:
            payload.update(platform.format_payload(push_message))
        payload["default"] = push_message.message
        json_string = json.dumps(payload)
        return connection.publish(message=json_string, topic=self.arn, message_structure="json")


class SNSNotCreatedException(Exception):
    message = "Register the instance before deleting it by calling register()"


class PushMessage(models.Model):
    MESSAGE_TYPE_DEFAULT = 0
    MESSAGE_TYPE_TOPIC = 1
    sound = models.TextField(blank=True, null=True)
    message = models.TextField(default='', null=True)
    has_new_content = models.BooleanField(default=False)
    context_id = models.TextField(default='none', null=True)
    context = models.TextField(default='default', null=True)
    badge_count = models.SmallIntegerField(default=0)
    extra_payload = models.TextField(blank=True, null=True)
    receiver_arn = models.TextField(blank=True, null=True)
    message_type = models.PositiveSmallIntegerField(default=0)

    # def __init__(self, message='', context='default', context_id='none', badge_count=0, sound=None, has_new_content=0,
    # extra_payload=None):
    # super(PushMessage, self).__init__(message, context, context_id, badge_count, sound, has_new_content,
    # extra_payload)
    #     self.sound = sound
    #     self.message = message
    #     self.has_new_content = has_new_content
    #     self.context_id = context_id
    #     self.context = context
    #     self.badge_count = badge_count
    #     self.extra_payload = extra_payload

    def as_dict(self):
        d = {'message': self.message,
             'context': self.context,
             'context_id': self.context_id,
             'badge_count': self.badge_count,
             'sound': self.sound,
             'has_new_content': self.has_new_content}
        if self.extra_payload:
            d.update(self.extra_payload)
        return d


def format_push(badgeCount, context, context_id, has_new_content, message, sound):
    if message:
        message = trim_message(message)

    payload = {
        'aps': {
            "content-available": has_new_content,
        },
        "ctx": context,
        "id": context_id
    }

    if message and len(message) > 0:
        payload['aps']['alert'] = message

    if not badgeCount is None:
        payload['aps'].update({
            "badge": badgeCount,
        })

    if not sound is None:
        payload['aps'].update({
            'sound': sound,
        })

    return payload


def trim_message(message):
    import sys

    if sys.getsizeof(message) > 140:
        while sys.getsizeof(message) > 140:
            message = message[:-3]
        message += '...'
    return message



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'PushMessage.context'
        db.alter_column('scarface_pushmessage', 'context', self.gf('django.db.models.fields.TextField')(null=True))

        # Changing field 'PushMessage.context_id'
        db.alter_column('scarface_pushmessage', 'context_id', self.gf('django.db.models.fields.TextField')(null=True))

    def backwards(self, orm):

        # Changing field 'PushMessage.context'
        db.alter_column('scarface_pushmessage', 'context', self.gf('django.db.models.fields.TextField')())

        # Changing field 'PushMessage.context_id'
        db.alter_column('scarface_pushmessage', 'context_id', self.gf('django.db.models.fields.TextField')())

    models = {
        'scarface.pushmessage': {
            'Meta': {'object_name': 'PushMessage'},
            'badge_count': ('django.db.models.fields.SmallIntegerField', [], {'default': '0'}),
            'context': ('django.db.models.fields.TextField', [], {'null': 'True', 'default': "'default'"}),
            'context_id': ('django.db.models.fields.TextField', [], {'null': 'True', 'default': "'none'"}),
            'extra_payload': ('django.db.models.fields.TextField', [], {'blank': 'True', 'null': 'True'}),
            'has_new_content': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'message': ('django.db.models.fields.TextField', [], {'default': "''"}),
            'message_type': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'receiver_arn': ('django.db.models.fields.TextField', [], {'blank': 'True', 'null': 'True'}),
            'sound': ('django.db.models.fields.TextField', [], {'blank': 'True', 'null': 'True'})
        }
    }

    complete_apps = ['scarface']


# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'PushMessage'
        db.create_table(u'scarface_pushmessage', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('sound', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('message', self.gf('django.db.models.fields.TextField')(default='')),
            ('has_new_content', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('context_id', self.gf('django.db.models.fields.TextField')(default='none')),
            ('context', self.gf('django.db.models.fields.TextField')(default='default')),
            ('badge_count', self.gf('django.db.models.fields.SmallIntegerField')(default=0)),
            ('extra_payload', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('receiver_arn', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('message_type', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
        ))
        db.send_create_signal(u'scarface', ['PushMessage'])


    def backwards(self, orm):
        # Deleting model 'PushMessage'
        db.delete_table(u'scarface_pushmessage')


    models = {
        u'scarface.pushmessage': {
            'Meta': {'object_name': 'PushMessage'},
            'badge_count': ('django.db.models.fields.SmallIntegerField', [], {'default': '0'}),
            'context': ('django.db.models.fields.TextField', [], {'default': "'default'"}),
            'context_id': ('django.db.models.fields.TextField', [], {'default': "'none'"}),
            'extra_payload': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'has_new_content': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'message': ('django.db.models.fields.TextField', [], {'default': "''"}),
            'message_type': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'receiver_arn': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'sound': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['scarface']


"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""
from boto.exception import BotoServerError
from django.test import TestCase
from django.test.utils import override_settings
from six import StringIO
from .models import APNApplication, GCMApplication, Topic, SNSDevice, PushMessage
from .utils import get_sns_connection, DefaultConnection, APP_PREFIX
from django.core.management import call_command


class SNSAppManagement(TestCase):
    def setUp(self):
        self.app_name = APP_PREFIX
        self.resources = list()
        self.gcm_token = "APA91bEUOyAVwExNEL1HPbWRq1lNKWGm5v4agR0idJYghHshF59MTZ9zOYiIeJCL0zu2_m5vPxEwieR3gOih225kI42MO-Np239-L1QLxrOfSLJtzfQf7oI_SnO_ekmM6kPCcMPUlx5xQ6LU3vEZKhy-GpaQTgFfCA"
        self.apn_token = "40e2aca271e95cd974873d62bc7360c3e0b94e66194b350ad0b9cba4ed6ddcdb"

    @property
    def topic_name(self):
        return self.app_name + "topic"

    def tearDown(self):
        gcm_application = GCMApplication(self.app_name)
        gcm_application.register()
        self.resources.extend(gcm_application.all_devices())
        self.resources.append(gcm_application)
        for resource in self.resources:
            resource.delete()

    def platform_test(self, apn_platform):
        result = apn_platform.register()
        self.resources.append(apn_platform)
        return result

    def create_apn_device(self):
        app = APNApplication(self.app_name)
        app.register()
        device = SNSDevice(app, self.apn_token)
        created = device.register()
        self.resources.append(device)
        return created, device

    def create_gcm_device(self):
        gcm_application = GCMApplication(self.app_name)
        gcm_application.register()
        device = SNSDevice(gcm_application, self.gcm_token)
        created = device.register()
        self.resources.append(device)
        return created, device

    def test_add_apn(self):
        """
        Test adding APN app
        """
        apn_platform = APNApplication(self.app_name)
        result = self.platform_test(apn_platform)
        self.assertTrue(result)

    def test_add_gcm(self):
        result = self.platform_test(GCMApplication(self.app_name))
        self.assertTrue(result)

    def test_delete_gcm(self):
        gcm_application = GCMApplication(self.app_name)
        gcm_application.register()
        result = gcm_application.delete()
        self.assertTrue(result)

    def test_delete_does_not_exist(self):
        gcm_application = GCMApplication('test_no_app')
        gcm_application.arn = 'arn:test_no_app'
        self.assertRaises(BotoServerError, gcm_application.delete)

    def create_app_topic(self):
        topic = Topic(self.topic_name)
        success = topic.register()
        self.resources.append(topic)
        return success, topic

    def test_add_topic(self):
        success, topic = self.create_app_topic()
        self.assertTrue(topic)

    def test_delete_topic(self):
        topic = Topic(self.topic_name)
        topic.register()
        result = topic.delete()
        self.assertTrue(result)

    def test_add_gcm_device(self):
        device = self.create_gcm_device()
        self.assertTrue(device)

    def test_add_apn_device(self):
        created, device = self.create_apn_device()
        self.assertTrue(created)

    def test_list_devices(self):
        gcm_application = GCMApplication(self.app_name)
        gcm_application.register()
        created, device = self.create_gcm_device()
        devices = gcm_application.all_devices()
        self.assertListEqual(devices, [device, ])

    def test_send_gcm_push(self):
        created, device = self.create_gcm_device()
        success = device.send(
            PushMessage(badge_count=1, context='url_alert', context_id='none',
                        has_new_content=True, message="Hello world!", sound="default"))
        self.assertEqual(PushMessage.objects.all().count(), 1)

    def test_send_apn_push(self):
        created, device = self.create_apn_device()
        message = PushMessage(badge_count=1, context='url_alert', context_id='none', has_new_content=True,
                              message="Hello world!", sound="default")
        device.send(message)
        self.assertEqual(PushMessage.objects.all().count(), 1)

    def test_add_to_app_topic(self):
        created_apn, apn_device = self.create_apn_device()
        created_gcm, gcm_device = self.create_gcm_device()
        topic = gcm_device.platform.app_topic
        subscriptions = topic.all_subscriptions()
        gcm_devices = gcm_device.platform.all_devices()
        apn_devices = apn_device.platform.all_devices()
        gcm_devices.extend(apn_devices)
        self.assertEqual(len(subscriptions), len(gcm_devices))

    def test_send_to_topic(self):
        created, apn_device = self.create_apn_device()
        created_gcm, gcm_device = self.create_gcm_device()
        topic = gcm_device.platform.app_topic
        message = PushMessage(badge_count=1, context='url_alert', context_id='none', has_new_content=True,
                              message="Hello Topic!", sound="default")
        topic.send(message, platforms=[gcm_device.platform, apn_device.platform])
        self.assertEqual(PushMessage.objects.all().count(), 1)

    def test_update_with_same_token(self):
        created, device = self.create_apn_device()
        device_updated = device.update(self.apn_token)
        self.assertIsNotNone(device_updated)

    def test_register_with_other_user_data(self):
        created, device = self.create_apn_device()
        self.assertRaises(BotoServerError, device.register, {"custom_user_data": "Hello world!"})

    def test_update_with_other_user_data(self):
        created, device = self.create_apn_device()
        result = device.update(custom_user_data="Hello world!")
        self.assertIsNotNone(result)

    def test_register_or_update(self):
        created, device = self.create_apn_device()
        result = device.register_or_update(custom_user_data="Hello world!")
        self.assertIsNotNone(result)

    @override_settings(SCARFACE_LOGGING_ENABLED=False)
    def test_disable_logging(self):
        created, device = self.create_gcm_device()
        device.send(PushMessage(badge_count=1, context='url_alert', context_id='none', has_new_content=True,
                              message="Hello Topic!", sound="default"))
        self.assertEqual(0,PushMessage.objects.all().count())

class DefaultConnectionWrapperTestCase(TestCase):
    def test_wrapper_empty(self):
        a, connection = connection_test()
        self.assertIsNotNone(connection)

    def test_wrapper_first_is_set_as_arg(self):
        a_input = 123
        a, connection = connection_test(a_input)
        self.assertIsNotNone(connection)
        self.assertEqual(a_input, a)

    def test_wrapper_first_is_set_as_kwarg(self):
        a_input = 123
        a, connection = connection_test(a=a_input)
        self.assertIsNotNone(connection)
        self.assertEqual(a_input, a)

    def test_wrapper_connection_is_set_as_arg(self):
        a_input = 123
        connection_input = get_sns_connection()
        a, connection = connection_test(a_input, connection_input)
        self.assertTrue(connection_input is connection)
        self.assertEqual(a_input, a)

    def test_wrapper_connection_is_set_as_kwarg(self):
        a_input = 123
        connection_input = get_sns_connection()
        a, connection = connection_test(a_input, connection=connection_input)
        self.assertTrue(connection_input is connection)
        self.assertEqual(a_input, a)

    def test_wrapper_connection_is_set_none(self):
        a_input = 123
        a, connection = connection_test(a_input, connection=None)
        self.assertIsNotNone(connection)
        self.assertEqual(a_input, a)

    def test_wrapper_connection_is_set_none_arg(self):
        a_input = 123
        a, connection = connection_test(a_input, None)
        self.assertIsNotNone(connection)
        self.assertEqual(a_input, a)


@DefaultConnection
def connection_test(a=None, connection=None):
    return a, connection


class ExtractKeysCommand(TestCase):
    def test_command_output(self):
        out = StringIO()
        call_command("extract_keys", file="local_push.p12", password="bazinga", stdout=out)
        out_getvalue = out.getvalue()
        self.assertIn('SCARFACE_APNS_CERTIFICATE', out_getvalue)


__author__ = 'dreipol GmbH'
from django.conf import settings
from boto import sns
from functools import partial
import inspect


class Decorator(object):
    def __init__(self, function, *args, **kwargs):
        self.function = function

    @property
    def original_function(self):
        function = self.function
        while isinstance(function, Decorator):
            function = function.function
        return function


class DefaultConnection(Decorator):
    def __get__(self, obj, objtype=None):
        if obj is None:
            return self.function
        return partial(self, obj)

    def __call__(self, *args, **kwargs):
        connection_keyword = 'connection'
        if len(args) + len(kwargs) == 0:
            call_kwargs = dict()
        else:
            call_kwargs = inspect.getcallargs(self.original_function, *args, **kwargs)
        if not call_kwargs.get(connection_keyword, None):
            call_kwargs[connection_keyword] = get_sns_connection()
        return self.function(**call_kwargs)


class PushLogger(Decorator):
    def __get__(self, obj, objtype=None):
        if obj is None:
            return self.function
        self.obj = obj
        return partial(self, obj)

    def __call__(self, *args, **kwargs):
        call_kwargs = inspect.getcallargs(self.original_function, *args, **kwargs)
        push_message = call_kwargs.get('push_message')
        self.obj.sign(push_message)
        if logging_enabled():
            push_message.save()
        return self.function(*args, **kwargs)


def get_sns_connection(region='eu-west-1'):
    """
    Creates a new AWS connection based upon the credentials defined in the django configuration
    :param region: the region of the DynamoDB, defaults to Ireland
    :return: a new dynamodb2 connection
    """
    return sns.connect_to_region(region, aws_access_key_id=settings.AWS_ACCESS_KEY,
                                 aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY)

APP_PREFIX = settings.SCARFACE_SNS_APP_PREFIX if hasattr(settings, 'SCARFACE_SNS_APP_PREFIX') else settings.SITE_NAME

def logging_enabled():
    return settings.SCARFACE_LOGGING_ENABLED if hasattr(settings, 'SCARFACE_LOGGING_ENABLED') else True